--[[

	Materialize color palette
	Wesley (kbmonkey) 2018

	Colors from materializecss
	Licensed under MIT
	https://materializecss.com/color.html

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above
     copyright notice, this list of conditions and the following disclaimer
     in the documentation and/or other materials provided with the
     distribution.
   * Neither the name of the  nor the names of its
     contributors may be used to endorse or promote products derived from
     this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

]]--

local colors = require ("colors")
local screenwidth, screenheight = love.graphics.getDimensions ()
local gridsize = 36
local grid = { }
local canvas = nil
local fpslimit = 1/5
local limit = true

function love.load ()

	love.graphics.setFont (love.graphics.newFont (20) )

	-- load colors into indexed table
	for name, value in pairs (colors) do
		local cell = {
			name = name,
			value = value
		}
		table.insert (grid, cell)
	end

	-- sort by name
	table.sort (grid, function (a, b) return a.name < b.name end)

	-- wrap position to new lines on these keys
	local wrapon = {
		["purple"] = true,
		["pink"] = true,
		["red"] = true,
		["blue"] = true,
		["deep-purple"] = true,
		["indigo"] = true,
		["light-blue"] = true,
		["cyan"] = true,
		["teal"] = true,
		["green"] = true,
		["lime"] = true,
		["light-green"] = true,
		["yellow"] = true,
		["orange"] = true,
		["amber"] = true,
		["deep-orange"] = true,
		["grey"] = true,
		["brown"] = true,
		["blue-grey"] = true,
		["black"] = true,
		["white"] = true
	}

	-- position on screen
	local x, y = 0, -gridsize

	for _, cell in ipairs (grid) do

		if wrapon[cell.name] then
			x = 0
			y = y + gridsize
		end

		cell.x = x
		cell.y = y
		cell.width = gridsize
		cell.height = gridsize
		cell.right = cell.x + gridsize
		cell.bottom = cell.y + gridsize

		-- wrap to next position
		--~ x = x + gridsize
		--~ if x > (screenwidth - gridsize) then
			--~ x = 0
			--~ y = y + gridsize
		--~ end

		-- move position on key match
		x = x + gridsize
	end

end

function love.update (dt)
   if limit and dt < fpslimit then
      love.timer.sleep(fpslimit - dt)
   end
end

function love.keypressed (key)
	if key == "escape" then
		love.event.quit ()
	end
end

function love.draw ()

	if not canvas then

		-- render cells to a canvas the first time
		canvas = love.graphics.newCanvas ()
		love.graphics.setCanvas (canvas)

		-- background
		love.graphics.clear (colors["blue-grey"])

		-- fill cells
		for _, cell in ipairs (grid) do
			love.graphics.setColor (cell.value)
			love.graphics.rectangle ("fill", cell.x, cell.y, cell.width, cell.height)
		end

		-- cell borders
		love.graphics.setColor (color.white)
		for _, cell in ipairs (grid) do
			love.graphics.rectangle ("line", cell.x, cell.y, cell.width, cell.height)
		end

		-- restore canvast
		love.graphics.setCanvas ()
	end

	-- draw the cells canvas
	love.graphics.setColor (colors.white)
	love.graphics.draw (canvas)

	if grid.selected then
		love.graphics.setColor (colors["amber lighten-4"])
		love.graphics.printf (grid.selected, 600, 10, 400, "left")
	end
	if grid.copied then
		love.graphics.setColor (colors["indigo lighten-4"])
		love.graphics.printf (grid.copied, 600, 40, 400, "left")
	end

end

function love.mousepressed (x, y, button, istouch, presses)
	if grid.selected then
		love.system.setClipboardText (grid.selected)
		grid.copied = string.format ("copied %q to clipboard", grid.selected)
	end
end

function love.mousereleased (x, y, button, istouch, presses)

end

function love.mousemoved (x, y, dx, dy, istouch)
	grid.selected = nil
	grid.copied = nil
	for _, cell in ipairs (grid) do
		if not grid.selected then
			if x > cell.x and x < cell.right then
				if y > cell.y and y < cell.bottom then
					grid.selected = cell.name
				end
			end
		end
	end
end
